/*
	Software created for Digilent Contest 2016
	Project Name: Thermovision Camera
	Library used: Socket, OpenCV
	Revision: 2.2
	Client.cpp
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/wait.h>
#include <signal.h>
#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/opencv.hpp>
#include "flir_grading.h"

#define	SOCKET_PORT				7521
#define	SOCKET_SERVER			"127.0.0.1"
#define SOCKET_MAXDATASIZE		4800
#define SOCKET_BACKLOG		  10

using namespace cv;
using namespace std;

int main(int argc, char *argv[]) {
	int socket_sockfd;

	int socket_port;
	int socket_data;
	struct sockaddr_in server_addr;
	struct hostent *socket_server;

	//Define the frame buffers, each 1/4 of 4800 because the maximum dimension of a packet is around 1500 bytes; 1200 will work just fine
	uint8_t socket_buffer_a[SOCKET_MAXDATASIZE];

	uint8_t	socket_buffer_loses[6] = {1, 1, 1, 1, 1, 1};
	uint8_t socket_buffer_complete[6] = {0, 0, 0, 0, 0, 0};

	//Temperatures buffer to display them later on the screen
	uint16_t socket_temperatures[6];	//To receive a temperature is required 
	float lepton_temp_min;
	float lepton_temp_max;
	float lepton_temp_center;
	char display_min_temp[10];
	char display_max_temp[10];
	char display_center_temp[10];
	
	//OpenCV Stuffs
	Mat lepton_image = imread("template.png");		  //The image is used to define a certain dimension of the matrix; in our case is 80x60
	Mat scale_graph = imread("graph.png");			  //"Legend" for temperature range
	Mat resized_image;								  //Matrix for resized flir-lepton image
	Mat camera_image;								   //Here will be "stored" the image from camera
	Mat output_image;								   //Image which will be diplayed on the screen
	Size received_imageresize(640, 480);				//Define the desired size of image
	VideoCapture video_camera(0);					   //Open the main camera port; port "0"

	printf("\n\n");

	//If no arguments are introduced set the "default" host and port.
	if (argc < 3) {
		printf("USAGE: hostname port\nInitializing connection with default parameters:\n  HOST: 127.0.0.1\n  PORT: 7521\n\n");
		socket_port = SOCKET_PORT;
	} else {
		//Convert to int the arg[2] (the port)
		socket_port = atoi(argv[2]);
	}

	//Create socket
	socket_sockfd = socket(AF_INET, SOCK_STREAM, 0);	//The sockets returns a "valid" number

	//Check if the socket can be opened
	if (socket_sockfd < 0) {printf("ERROR: The socket cannot be opened\nExiting...\n\n"); exit(1);}

	//Set the server
	if (argc < 3) {
		//Default server defined above
		socket_server = gethostbyname(SOCKET_SERVER);
	} else {
		//Server inserted by user
		socket_server = gethostbyname(argv[1]);
	}

	if (socket_server == NULL) {printf("ERROR: No such server\nExiting...\n\n"); exit(1);}

	//Set all arguments to "0"
	bzero((char*) &server_addr, sizeof(server_addr));

	//Set the connection type, the port and server address
	server_addr.sin_family = AF_INET;
	server_addr.sin_port = htons(SOCKET_PORT);
	server_addr.sin_addr = *((struct in_addr *)socket_server->h_addr);
	memset(&(server_addr.sin_zero), '\0', 8);

	//Connect to server and check the connection
	if (connect(socket_sockfd,(struct sockaddr *) &server_addr, sizeof(server_addr)) < 0) {printf("ERROR: Connection failed.\nExiting...\n\n"); exit(1);}

	//Create a window using OpenCV with name "Flir Lepton"
	namedWindow( "Flir Lepton", 1);

	int state = 0;

	//Get the image from camera and store in Mat
	

	//Load the color mapping into "local" buffer to give a little bit of color to our image
	const int *color_grading = flir_grading;
	
while(1) {
		video_camera >> camera_image;
		int socket_buffer_state = 0;
			while(socket_buffer_state < 2) {
				int counter = 0;
				switch(socket_buffer_state){
					case 0:{
						//-------------------------------------------------RECEIVE THE BUFFER A-----------------------------------------------------------------------
						counter = 0;		//Here is the buffer counter to "convert" the single-line buffer into matrix
						socket_data = recv(socket_sockfd, socket_buffer_a, (SOCKET_MAXDATASIZE), MSG_WAITALL); //Return the number with received data
						//Check if the data has been received;
						if (socket_data < 0) {printf("ERROR: Failed to receive data from server\nExiting...\n\n"); exit(1);}
						
						//Fill the OpenCV Mat from row 0 up to 14 with data from buffer
						for (int row = 0; row < 60; row++) {
							for (int col = 0; col < 80; col++) {
									lepton_image.at<Vec3b>(Point(col, row))[2] = color_grading[(3*socket_buffer_a[counter])+2];  //Value for pixel R
									lepton_image.at<Vec3b>(Point(col, row))[1] = color_grading[(3*socket_buffer_a[counter])+1];  //Value for pixel G
									lepton_image.at<Vec3b>(Point(col, row))[0] = color_grading[(3*socket_buffer_a[counter])];  //Value for pixel B
									counter++;
							}
						}
						//-------------------------------------------------END OF BUFFER A-----------------------------------------------------------------------
					}
					break;
					case 1:{
						//-------------------------------------------------START OF TEMP READING-----------------------------------------------------------------------
						//Receive the min, max and center temperatures
						socket_data = recv(socket_sockfd, socket_temperatures, 12, MSG_WAITALL); //Return the number with received data
						//Check if the data has been received
						if (socket_data < 0) {printf("ERROR: Failed to receive data from server\nExiting...\n\n"); exit(1);}
						//-------------------------------------------------END OF TEMP READING-----------------------------------------------------------------------
					}
					break;
				}
				socket_buffer_state++;
			}
			
				resize(lepton_image, camera_image, received_imageresize); //Resize the image to desired resolution(640x480)
				addWeighted(resized_image, 0.65, camera_image, 0.35, 0.0, output_image); //Set opacity at 65% for lepton and 35% for camera
				scale_graph.copyTo(output_image(Rect(615, 40, 22, 400))); //Place the bar on the right side of the screen
				
				//"Convert" the received temperatures into float
				lepton_temp_min = ((float)socket_temperatures[1])/10;	//Lowest temperature from screen
				lepton_temp_max = ((float)socket_temperatures[0])/10;	//Highest temperature from screen
				lepton_temp_center = ((float)socket_temperatures[2])/10;//Center temperature

				//Convert to string the temp. values
				snprintf(display_min_temp, sizeof(display_min_temp), "%.1f", lepton_temp_min);
				snprintf(display_max_temp, sizeof(display_max_temp), "%.1f", lepton_temp_max);
				snprintf(display_center_temp, sizeof(display_center_temp), "%.1f", lepton_temp_center);

				//Display the temperatures on the screen (still in development, we wrote some numbers to align the text)
				putText(output_image, display_max_temp, Point(580, 30), FONT_HERSHEY_PLAIN, 1.5, CV_RGB(255, 255 ,255), 4.0);
				putText(output_image, display_min_temp, Point(580, 465), FONT_HERSHEY_PLAIN, 1.5, CV_RGB(255, 255 ,255), 4.0);
				putText(output_image, display_center_temp, Point(10, 25), FONT_HERSHEY_PLAIN, 1.5, CV_RGB(255, 255 ,255), 4.0);

				//Display the image in "Flir Lepton" Window defined earlier
				imshow( "Flir Lepton", output_image);

				//If a key was pressed "break" the while loop, close the socket and exit
				if (waitKey(30) >= 0) {break;exit(1);}
}

	//Close the socket
	close(socket_sockfd);

	return 0;
	
}
