/*
    Software created for Digilent Contest 2016
    Project Name: Thermovision Camera
    Library used: Socket, OpenCV
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/wait.h>
#include <signal.h>
#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/opencv.hpp>
#include "flir_grading.h"

#define	SOCKET_PORT				7521
#define	SOCKET_SERVER			"127.0.0.1"
#define SOCKET_MAXDATASIZE		4800
#define SOCKET_BACKLOG          10

using namespace cv;
using namespace std;

int main(int argc, char *argv[]) {
    int socket_sockfd;

	int socket_port;
	int socket_data;
	struct sockaddr_in server_addr;
	struct hostent *socket_server;

    //Define the frame buffers, each 1/4 of 4800 because the maximum dimension of a packet is around 1500 bytes; 1200 will work just fine
    uint8_t socket_buffer_a[SOCKET_MAXDATASIZE/4];
    uint8_t socket_buffer_b[SOCKET_MAXDATASIZE/4];
    uint8_t socket_buffer_c[SOCKET_MAXDATASIZE/4];
    uint8_t socket_buffer_d[SOCKET_MAXDATASIZE/4];

	//OpenCV Stuffs
    Mat lepton_image = imread("template.png");          //The image is used to define a certain dimension of the matrix; in our case is 80x60
    Mat scale_graph = imread("graph.png");              //"Legend" for temperature range
    Mat resized_image;                                  //Matrix for resized flir-lepton image
    Mat camera_image;                                   //Here will be "stored" the image from camera
    Mat output_image;                                   //Image which will be diplayed on the screen
    Size received_imageresize(640, 480);                //Define the desired size of image
    VideoCapture video_camera(0);                       //Open the main camera port; port "0"

	printf("\n\n");

    //If no arguments are introduced set the "default" host and port.
    if (argc < 3) {
        printf("USAGE: hostname port\nInitializing connection with default parameters:\n  HOST: 127.0.0.1\n  PORT: 7521\n\n");
        socket_port = SOCKET_PORT;
    } else {
        //Convert to int the arg[2] (the port)
        socket_port = atoi(argv[2]);
    }

    //Create socket
    socket_sockfd = socket(AF_INET, SOCK_STREAM, 0);    //The sockets returns a "valid" number

    //Check if the socket can be opened
    if (socket_sockfd < 0) {printf("ERROR: The socket cannot be opened\nExiting...\n\n"); exit(1);}

    //Set the server
    if (argc < 3) {
        //Default server defined above
        socket_server = gethostbyname(SOCKET_SERVER);
    } else {
        //Server inserted by user
        socket_server = gethostbyname(argv[1]);
    }

    if (socket_server == NULL) {printf("ERROR: No such server\nExiting...\n\n"); exit(1);}

    //Set all arguments to "0"
    bzero((char*) &server_addr, sizeof(server_addr));

    //Set the connection type, the port and server address
    server_addr.sin_family = AF_INET;
    server_addr.sin_port = htons(SOCKET_PORT);
    server_addr.sin_addr = *((struct in_addr *)socket_server->h_addr);
    memset(&(server_addr.sin_zero), '\0', 8);

    //Connect to server and check the connection
    if (connect(socket_sockfd,(struct sockaddr *) &server_addr, sizeof(server_addr)) < 0) {printf("ERROR: Connection failed.\nExiting...\n\n"); exit(1);}

    //Create a window using OpenCV with name "Flir Lepton"
    namedWindow( "Flir Lepton", 1);

    int state = 0;

    //Get the image from camera and store in Mat
    video_camera.read(camera_image);

    //int frame_counter = 0; // Used to check the packets frames received
    while(1) {
    for(int k = 0; k < 8; k++) {
        //We are using a "for" loop to collect packets from socket one by one
        for(int i = 0; i < 4; i++){
            switch(i){

                case 0:
                    //Receive the buffer A
                    socket_data = recv(socket_sockfd, socket_buffer_c, (SOCKET_MAXDATASIZE/4), 0); //Return the number with received data
                    //Check if the data has been received;
                    if (socket_data < 0) {printf("ERROR: Failed to receive data from server\nExiting...\n\n"); exit(1);}
                    //printf("-----FRAME %d-----\nSocket Buffer A: %d\n", frame_counter, socket_data); //Debugging stuffs
                    break;

                case 1:
                    //Receive the buffer B
                    socket_data = recv(socket_sockfd, socket_buffer_d, (SOCKET_MAXDATASIZE/4), 0); //Return the number with received data
                    //Check if the data has been received
                    if (socket_data < 0) {printf("ERROR: Failed to receive data from server\nExiting...\n\n"); exit(1);}
                    //printf("Socket Buffer B: %d\n", socket_data);
                    break;

                case 2:
                    //Receive the buffer C
                    socket_data = recv(socket_sockfd, socket_buffer_a, (SOCKET_MAXDATASIZE/4), 0); //Return the number with received data
                    //Check if the data has been received
                    if (socket_data < 0) {printf("ERROR: Failed to receive data from server\nExiting...\n\n"); exit(1);}
                    //printf("Socket Buffer C: %d\n", socket_data);
                    break;

                case 3:
                    //Receive the buffer D
                    socket_data = recv(socket_sockfd, socket_buffer_b, (SOCKET_MAXDATASIZE/4), 0); //Return the number with received data
                    //Check if the data has been received
                    if (socket_data < 0) {printf("ERROR: Failed to receive data from server\nExiting...\n\n"); exit(1);}
                    //printf("Socket Buffer D: %d\n--------------------\n", socket_data);
                    break;
            }
        }


                //Fill the defined OpenCV Matrix with data from buffers
                //There are many others ways to fill the matrix, but I choosed this "long" way

                //Load the color mapping into "local" buffer to give a little bit of color to our image
                const int *color_grading = flir_grading;

                //Fill the matrix with first 15 rows
                int counter = 0;        //Here is the buffer counter to "convert" the single-line buffer into matrix
                for (int row = 0; row < 15; row++) {
                    for (int col = 0; col < 80; col++) {
                            lepton_image.at<Vec3b>(Point(col, row))[2] = color_grading[(3*socket_buffer_a[counter])+2];  //Value for pixel R
                            lepton_image.at<Vec3b>(Point(col, row))[1] = color_grading[(3*socket_buffer_a[counter])+1];  //Value for pixel G
                            lepton_image.at<Vec3b>(Point(col, row))[0] = color_grading[(3*socket_buffer_a[counter])];  //Value for pixel B
                            counter++;
                    }
                }

                //Reset the buffer counter and fill the matrix between rows 16 and 30
                counter = 0;
                for (int row = 15; row < 30; row++) {
                    for (int col = 0; col < 80; col++) {
                            lepton_image.at<Vec3b>(Point(col, row))[2] = color_grading[(3*socket_buffer_b[counter])+2];  //Value for pixel R
                            lepton_image.at<Vec3b>(Point(col, row))[1] = color_grading[(3*socket_buffer_b[counter])+1];  //Value for pixel G
                            lepton_image.at<Vec3b>(Point(col, row))[0] = color_grading[(3*socket_buffer_b[counter])];  //Value for pixel B
                            counter++;
                    }
                }


                counter = 0;
                for (int row = 30; row < 45; row++) {
                    for (int col = 0; col < 80; col++) {
                            lepton_image.at<Vec3b>(Point(col, row))[2] = color_grading[(3*socket_buffer_c[counter])+2];  //Value for pixel R
                            lepton_image.at<Vec3b>(Point(col, row))[1] = color_grading[(3*socket_buffer_c[counter])+1];  //Value for pixel G
                            lepton_image.at<Vec3b>(Point(col, row))[0] = color_grading[(3*socket_buffer_c[counter])];  //Value for pixel B
                            counter++;
                    }
                }

                counter = 0;
                for (int row = 45; row < 60; row++) {
                    for (int col = 0; col < 80; col++) {
                            lepton_image.at<Vec3b>(Point(col, row))[2] = color_grading[(3*socket_buffer_d[counter])+2];  //Value for pixel R
                            lepton_image.at<Vec3b>(Point(col, row))[1] = color_grading[(3*socket_buffer_d[counter])+1];  //Value for pixel G
                            lepton_image.at<Vec3b>(Point(col, row))[0] = color_grading[(3*socket_buffer_d[counter])];  //Value for pixel B
                            counter++;
                    }
                }
        }

                resize(lepton_image, resized_image, received_imageresize); //Resize the image to desired resolution(640x480)
                addWeighted(resized_image, 0.65, camera_image, 0.35, 0.0, output_image); //Set opacity at 65% for lepton and 35% for camera
                scale_graph.copyTo(output_image(Rect(615, 40, 22, 400))); //Place the bar on the right side of the screen

                //Display the temperatures on the screen
                putText(output_image, "10.5", Point(580, 30), FONT_HERSHEY_PLAIN, 1.5, CV_RGB(255, 255 ,255), 4.0);
                putText(output_image, "10.5", Point(580, 470), FONT_HERSHEY_PLAIN, 1.5, CV_RGB(255, 255 ,255), 4.0);

                //Display the image in "Flir Lepton" Window defined earlier
                imshow( "Flir Lepton", output_image);

                //If a key was pressed "break" the while loop, close the socket and exit
                if (waitKey(30) >= 0) {break;}
    }


    //Close the socket
	close(socket_sockfd);

	return 0;
	
}
